%--------------------------------------------------------------------------
%               Systems and control theory
%               Prof: Luigi Biagiotti
%               Solution of exercises #5
%--------------------------------------------------------------------------
clear all
close all
clc

%% 1st point.  Model definition and linearization
m = 0.1;
M = 1;
L = 1;
g = 9.81;
% Nonlinear model in the file CartPoleNonLinearModel.mdl
 
%% 2nd point.
X0 = [0 0 0 0]'; % Generic initial conditions
Xe = [1 0 0 0]';
ue = 0;

[A,B,C,D]=linmod('CartPoleNonLinearModel',Xe,ue)
eig(A)

%% 3rd point.  LQ Control
Q = diag([5,1]);
R = 1;

K = lqr(A,B,C'*Q*C,R);
disp('The Kalman gain is:') 
K

dX0 = [0 0 0.55 0]'; % Initial conditions for linearized system
sim('LinModel_LQ',6)
hf = figure;
set(hf,'name','Response of the (linearized) system with the LQ control')
subplot(2,1,1)
plot(time,dy(:,1))
grid on
ylabel('\delta x')
xlabel('time')
subplot(2,1,2)
plot(time,dy(:,2))
grid on
ylabel('\delta \theta')
xlabel('time')


%% 3rd point.  LQ Control with asymptotic state observer
no = rank(obsv(A,C)) % observability of the pair (A,C)

eig(A-B*K)
Lobs = - place(A',C',[-20 -20 -25 -25])'; % NOTE: it is not possible to use the acker function since there are two outputs
sim('LinModel_LQ_obs',6)
hf = figure;
set(hf,'name','Response of the (linearized) system with the LQ control and state observer')
subplot(2,1,1)
plot(time,dy(:,1))
grid on
ylabel('\delta x')
xlabel('time')
subplot(2,1,2)
plot(time,dy(:,2))
grid on
ylabel('\delta \theta')
xlabel('time')


%% 4th point.  LQ Control with asymptotic state observer applied to the
%% nonlinear system
X0 = Xe + dX0;
ye = [1 0]';
sim('CartPole_LQ_obs',6)
hf = figure;
set(hf,'name','Response of the nonlinear system with the LQ control and state observer')
subplot(2,1,1)
plot(time,y(:,1))
grid on
ylabel(' x')
xlabel('time')
subplot(2,1,2)
plot(time,y(:,2))
grid on
ylabel('\theta')
xlabel('time')

%% 5th point. Variation of the equilibrium point
% Definition of the variation law of the cart position
dt = 0.01;
t = 0:dt:20
t0=3;
t1=15;
T=t1-t0;
for i =1:length(t)
    if (t(i)<t0)
        Xe_t(i) = 0;
    elseif(t(i)<t1)
        tau = (t(i)-t0)/T;
        Xe_t(i) = 5*(3*tau^2-2*tau^3);
    else
        Xe_t(i) = 5;
    end
end
figure
plot(t,Xe_t)
grid on
xlabel('time')
ylabel('xe')

sim('CartPole_LQ_Obs_EquilibVariation',20)
hf = figure;
set(hf,'name','Response of the nonlinear system with the LQ control and state observer: equilibrium variation')
subplot(2,1,1)
plot(time,y_1)
grid on
hold on
plot(t,Xe_t,'r--')
ylabel('x')
xlabel('time')
subplot(2,1,2)
plot(time,y_2)
grid on
ylabel('\theta')
xlabel('time')
